#!/bin/bash

#
# List all instructions and their encoding.
#
list_insn()
{
    #
    # This is a plain copy/paste from "Table B-1 lists instructions by mnemonic"
    # from the "Signal Processing Engine (SPE) Programming Environments Manual,
    # Rev. 0" available at:
    # https://www.nxp.com/docs/en/reference-manual/SPEPEM.pdf
    #
    # We then do a tiny bit of substitution to make it easy to process with
    # `cut`.

    sed 's|/ / ||g;s|rD 0 0 0 0 0 r|rD /// r|g' <<EOF
brinc 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 0 1 1 1 1 EVX brinc
efdabs 0 0 0 1 0 0 rD rA /// 0 1 0 1 1 1 0 0 1 0 0 EFX efdabs
efdadd 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 1 0 0 0 0 0 EFX efdadd
efdcfs 0 0 0 1 0 0 rD 0 0 0 0 0 rB 0 1 0 1 1 1 0 1 1 1 1 EFX efdcfs
efdcfsf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 0 1 1 EFX efdcfsf
efdcfsi 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 0 0 1 EFX efdcfsi
efdcfuf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 0 1 0 EFX efdcfuf
efdcfui 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 0 0 0 EFX efdcfui
efdcmpeq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 1 0 1 1 1 0 EFX efdcmpeq
efdcmpgt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 1 0 1 1 0 0 EFX efdcmpgt
efdcmplt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 1 0 1 1 0 1 EFX efdcmplt
efdctsf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 1 1 1 EFX efdctsf
efdctsi 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 1 0 1 EFX efdctsi
efdctsiz 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 1 0 1 0 EFX efdctsiz
efdctuf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 1 1 0 EFX efdctuf
efdctui 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 0 1 0 0 EFX efdctui
efdctuiz 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 1 1 1 0 0 0 EFX efdctuiz
efddiv 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 1 0 1 0 0 1 EFX efddiv
efdmul 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 1 0 1 0 0 0 EFX efdmul
efdnabs 0 0 0 1 0 0 rD rA /// 0 1 0 1 1 1 0 0 1 0 1 EFX efdnabs
efdneg 0 0 0 1 0 0 rD rA /// 0 1 0 1 1 1 0 0 1 1 0 EFX efdneg
efdsub 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 1 0 0 0 0 1 EFX efdsub
efdtsteq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 1 1 1 1 1 0 EFX efdtsteq
efdtstgt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 1 1 1 1 0 0 EFX efdtstgt
efdtstlt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 1 1 1 1 0 1 EFX efdtstlt
efsabs 0 0 0 1 0 0 rD rA /// 0 1 0 1 1 0 0 0 1 0 0 EFX efsabs
efsadd 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 0 0 0 0 0 0 EFX efsadd
efscfd 0 0 0 1 0 0 rD 0 0 0 0 0 rB 0 1 0 1 1 0 0 1 1 1 1 EFX efscfd
efscfsf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 0 1 1 EFX efscfsf
efscfsi 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 0 0 1 EFX efscfsi
efscfuf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 0 1 0 EFX efscfuf
efscfui 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 0 0 0 EFX efscfui
efscmpeq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 0 0 1 1 1 0 EFX efscmpeq
efscmpgt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 0 0 1 1 0 0 EFX efscmpgt
efscmplt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 0 0 1 1 0 1 EFX efscmplt
efsctsf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 1 1 1 EFX efsctsf
efsctsi 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 1 0 1 EFX efsctsi
efsctsiz 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 1 0 1 0 EFX efsctsiz
efsctuf 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 1 1 0 EFX efsctuf
efsctui 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 0 1 0 0 EFX efsctui
efsctuiz 0 0 0 1 0 0 rD /// rB 0 1 0 1 1 0 1 1 0 0 0 EFX efsctuiz
efsdiv 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 0 0 1 0 0 1 EFX efsdiv
efsmul 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 0 0 1 0 0 0 EFX efsmul
efsnabs 0 0 0 1 0 0 rD rA /// 0 1 0 1 1 0 0 0 1 0 1 EFX efsnabs
efsneg 0 0 0 1 0 0 rD rA /// 0 1 0 1 1 0 0 0 1 1 0 EFX efsneg
efssub 0 0 0 1 0 0 rD rA rB 0 1 0 1 1 0 0 0 0 0 1 EFX efssub
efststeq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 0 1 1 1 1 0 EFX efststeq
efststgt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 0 1 1 1 0 0 EFX efststgt
efststlt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 1 0 1 1 1 0 1 EFX efststlt
evabs 0 0 0 1 0 0 rD rA /// 0 1 0 0 0 0 0 1 0 0 0 EVX evabs
evaddiw 0 0 0 1 0 0 rD UIMM rB 0 1 0 0 0 0 0 0 0 1 0 EVX evaddiw
evaddsmiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 1 0 0 1 EVX evaddsmiaaw
evaddssiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 0 0 0 1 EVX evaddssiaaw
evaddumiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 1 0 0 0 EVX evaddumiaaw
evaddusiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 0 0 0 0 EVX evaddusiaaw
evaddw 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 0 0 0 0 0 EVX evaddw
evand 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 0 0 0 1 EVX evand
evandc 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 0 0 1 0 EVX evandc
evcmpeq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 0 0 1 1 0 1 0 0 EVX evcmpeq
evcmpgts 0 0 0 1 0 0 crfD / / rA rB 0 1 0 0 0 1 1 0 0 0 1 EVX evcmpgts
evcmpgtu 0 0 0 1 0 0 crfD / / rA rB 0 1 0 0 0 1 1 0 0 0 0 EVX evcmpgtu
evcmplts 0 0 0 1 0 0 crfD / / rA rB 0 1 0 0 0 1 1 0 0 1 1 EVX evcmplts
evcmpltu 0 0 0 1 0 0 crfD / / rA rB 0 1 0 0 0 1 1 0 0 1 0 EVX evcmpltu
evcntlsw 0 0 0 1 0 0 rD rA /// 0 1 0 0 0 0 0 1 1 1 0 EVX evcntlsw
evcntlzw 0 0 0 1 0 0 rD rA /// 0 1 0 0 0 0 0 1 1 0 1 EVX evcntlzw
evdivws 0 0 0 1 0 0 rD rA rB 1 0 0 1 1 0 0 0 1 1 0 EVX evdivws
evdivwu 0 0 0 1 0 0 rD rA rB 1 0 0 1 1 0 0 0 1 1 1 EVX evdivwu
eveqv 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 1 0 0 1 EVX eveqv
evextsb 0 0 0 1 0 0 rD rA /// 0 1 0 0 0 0 0 1 0 1 0 EVX evextsb
evextsh 0 0 0 1 0 0 rD rA /// 0 1 0 0 0 0 0 1 0 1 1 EVX evextsh
evfsabs 0 0 0 1 0 0 rD rA /// 0 1 0 1 0 0 0 0 1 0 0 EVX evfsabs
evfsadd 0 0 0 1 0 0 rD rA rB 0 1 0 1 0 0 0 0 0 0 0 EVX evfsadd
evfscfsf 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 0 1 1 EVX evfscfsf
evfscfsi 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 0 0 1 EVX evfscfsi
evfscfuf 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 0 1 0 EVX evfscfuf
evfscfui 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 0 0 0 EVX evfscfui
evfscmpeq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 0 0 0 1 1 1 0 EVX evfscmpeq
evfscmpgt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 0 0 0 1 1 0 0 EVX evfscmpgt
evfscmplt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 0 0 0 1 1 0 1 EVX evfscmplt
evfsctsf 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 1 1 1 EVX evfsctsf
evfsctsi 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 1 0 1 EVX evfsctsi
evfsctsiz 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 1 0 1 0 EVX evfsctsiz
evfsctuf 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 1 1 0 EVX evfsctuf
evfsctui 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 0 1 0 0 EVX evfsctui
evfsctuiz 0 0 0 1 0 0 rD /// rB 0 1 0 1 0 0 1 1 0 0 0 EVX evfsctuiz
evfsdiv 0 0 0 1 0 0 rD rA rB 0 1 0 1 0 0 0 1 0 0 1 EVX evfsdiv
evfsmul 0 0 0 1 0 0 rD rA rB 0 1 0 1 0 0 0 1 0 0 0 EVX evfsmul
evfsnabs 0 0 0 1 0 0 rD rA /// 0 1 0 1 0 0 0 0 1 0 1 EVX evfsnabs
evfsneg 0 0 0 1 0 0 rD rA /// 0 1 0 1 0 0 0 0 1 1 0 EVX evfsneg
evfssub 0 0 0 1 0 0 rD rA rB 0 1 0 1 0 0 0 0 0 0 1 EVX evfssub
evfststeq 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 0 0 1 1 1 1 0 EVX evfststeq
evfststgt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 0 0 1 1 1 0 0 EVX evfststgt
evfststlt 0 0 0 1 0 0 crfD / / rA rB 0 1 0 1 0 0 1 1 1 0 1 EVX evfststlt
evldd 0 0 0 1 0 0 rD rA UIMM1 0 1 1 0 0 0 0 0 0 0 1 EVX evldd
evlddx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 0 0 0 0 0 EVX evlddx
evldh 0 0 0 1 0 0 rD rA UIMM 1 0 1 1 0 0 0 0 0 1 0 1 EVX evldh
evldhx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 0 0 1 0 0 EVX evldhx
evldw 0 0 0 1 0 0 rD rA UIMM 1 0 1 1 0 0 0 0 0 0 1 1 EVX evldw
evldwx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 0 0 0 1 0 EVX evldwx
evlhhesplat 0 0 0 1 0 0 rD rA UIMM2 0 1 1 0 0 0 0 1 0 0 1 EVX evlhhesplat
evlhhesplatx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 0 1 0 0 0 EVX evlhhesplatx
evlhhossplat 0 0 0 1 0 0 rD rA UIMM 2 0 1 1 0 0 0 0 1 1 1 1 EVX evlhhossplat
evlhhossplatx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 0 1 1 1 0 EVX evlhhossplatx
evlhhousplat 0 0 0 1 0 0 rD rA UIMM 2 0 1 1 0 0 0 0 1 1 0 1 EVX evlhhousplat
evlhhousplatx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 0 1 1 0 0 EVX evlhhousplatx
evlwhe 0 0 0 1 0 0 rD rA UIMM3 0 1 1 0 0 0 1 0 0 0 1 EVX evlwhe
evlwhex 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 1 0 0 0 0 EVX evlwhex
evlwhos 0 0 0 1 0 0 rD rA UIMM 3 0 1 1 0 0 0 1 0 1 1 1 EVX evlwhos
evlwhosx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 1 0 1 1 0 EVX evlwhosx
evlwhou 0 0 0 1 0 0 rD rA UIMM 3 0 1 1 0 0 0 1 0 1 0 1 EVX evlwhou
evlwhoux 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 1 0 1 0 0 EVX evlwhoux
evlwhsplat 0 0 0 1 0 0 rD rA UIMM 3 0 1 1 0 0 0 1 1 1 0 1 EVX evlwhsplat
evlwhsplatx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 1 1 1 0 0 EVX evlwhsplatx
evlwwsplat 0 0 0 1 0 0 rD rA UIMM 3 0 1 1 0 0 0 1 1 0 0 1 EVX evlwwsplat
evlwwsplatx 0 0 0 1 0 0 rD rA rB 0 1 1 0 0 0 1 1 0 0 0 EVX evlwwsplatx
evmergehi 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 1 1 0 0 EVX evmergehi
evmergehilo 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 1 1 1 0 EVX evmergehilo
evmergelo 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 1 1 0 1 EVX evmergelo
evmergelohi 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 1 1 1 1 EVX evmergelohi
evmhegsmfaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 1 0 1 0 1 1 EVX evmhegsmfaa
evmhegsmfan 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 1 0 1 0 1 1 EVX evmhegsmfan
evmhegsmiaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 1 0 1 0 0 1 EVX evmhegsmiaa
evmhegsmian 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 1 0 1 0 0 1 EVX evmhegsmian
evmhegumiaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 1 0 1 0 0 0 EVX evmhegumiaa
evmhegumian 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 1 0 1 0 0 0 EVX evmhegumian
evmhesmf 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 1 0 1 1 EVX evmhesmf
evmhesmfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 1 0 1 1 EVX evmhesmfa
evmhesmfaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 1 0 1 1 EVX evmhesmfaaw
evmhesmfanw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 1 0 1 1 EVX evmhesmfanw
evmhesmi 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 1 0 0 1 EVX evmhesmi
evmhesmia 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 1 0 0 1 EVX evmhesmia
evmhesmiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 1 0 0 1 EVX evmhesmiaaw
evmhesmianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 1 0 0 1 EVX evmhesmianw
evmhessf 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 0 0 1 1 EVX evmhessf
evmhessfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 0 0 1 1 EVX evmhessfa
evmhessfaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 0 0 1 1 EVX evmhessfaaw
evmhessfanw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 0 0 1 1 EVX evmhessfanw
evmhessiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 0 0 0 1 EVX evmhessiaaw
evmhessianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 0 0 0 1 EVX evmhessianw
evmheumi 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 1 0 0 0 EVX evmheumi
evmheumia 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 1 0 0 0 EVX evmheumia
evmheumiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 1 0 0 0 EVX evmheumiaaw
evmheumianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 1 0 0 0 EVX evmheumianw
evmheusiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 0 0 0 0 EVX evmheusiaaw
evmheusianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 0 0 0 0 EVX evmheusianw
evmhogsmfaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 1 0 1 1 1 1 EVX evmhogsmfaa
evmhogsmfan 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 1 0 1 1 1 1 EVX evmhogsmfan
evmhogsmiaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 1 0 1 1 0 1 EVX evmhogsmiaa
evmhogsmian 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 1 0 1 1 0 1 EVX evmhogsmian
evmhogumiaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 1 0 1 1 0 0 EVX evmhogumiaa
evmhogumian 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 1 0 1 1 0 0 EVX evmhogumian
evmhosmf 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 1 1 1 1 EVX evmhosmf
evmhosmfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 1 1 1 1 EVX evmhosmfa
evmhosmfaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 1 1 1 1 EVX evmhosmfaaw
evmhosmfanw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 1 1 1 1 EVX evmhosmfanw
evmhosmi 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 1 1 0 1 EVX evmhosmi
evmhosmia 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 1 1 0 1 EVX evmhosmia
evmhosmiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 1 1 0 1 EVX evmhosmiaaw
evmhosmianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 1 1 0 1 EVX evmhosmianw
evmhossf 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 0 1 1 1 EVX evmhossf
evmhossfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 0 1 1 1 EVX evmhossfa
evmhossfaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 0 1 1 1 EVX evmhossfaaw
evmhossfanw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 0 1 1 1 EVX evmhossfanw
evmhossiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 0 1 0 1 EVX evmhossiaaw
evmhossianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 0 1 0 1 EVX evmhossianw
evmhoumi 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 0 0 1 1 0 0 EVX evmhoumi
evmhoumia 0 0 0 1 0 0 rD rA rB 1 0 0 0 0 1 0 1 1 0 0 EVX evmhoumia
evmhoumiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 1 1 0 0 EVX evmhoumiaaw
evmhoumianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 1 1 0 0 EVX evmhoumianw
evmhousiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 0 0 0 0 1 0 0 EVX evmhousiaaw
evmhousianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 0 0 0 0 1 0 0 EVX evmhousianw
evmra 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 0 1 0 0 EVX evmra
evmwhsmf 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 0 1 1 1 1 EVX evmwhsmf
evmwhsmfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 0 1 1 1 1 EVX evmwhsmfa
evmwhsmi 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 0 1 1 0 1 EVX evmwhsmi
evmwhsmia 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 0 1 1 0 1 EVX evmwhsmia
evmwhssf 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 0 0 1 1 1 EVX evmwhssf
evmwhssfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 0 0 1 1 1 EVX evmwhssfa
evmwhumi 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 0 1 1 0 0 EVX evmwhumi
evmwhumia 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 0 1 1 0 0 EVX evmwhumia
evmwhusiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 0 0 1 0 0 EVX evmwhusiaaw
evmwhusianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 0 0 1 0 0 EVX evmwhusianw
evmwlumi 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 0 1 0 0 0 EVX evmwlumi
evmwlumia 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 0 1 0 0 0 EVX evmwlumia
evmwlumiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 0 1 0 0 0 EVX evmwlumiaaw
evmwlumianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 0 1 0 0 0 EVX evmwlumianw
evmwlusiaaw 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 0 0 0 0 0 EVX evmwlusiaaw
evmwlusianw 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 0 0 0 0 0 EVX evmwlusianw
evmwsmf 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 1 1 0 1 1 EVX evmwsmf
evmwsmfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 1 1 0 1 1 EVX evmwsmfa
evmwsmfaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 1 1 0 1 1 EVX evmwsmfaa
evmwsmfan 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 1 1 0 1 1 EVX evmwsmfan
evmwsmi 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 1 1 0 0 1 EVX evmwsmi
evmwsmia 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 1 1 0 0 1 EVX evmwsmia
evmwsmiaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 1 1 0 0 1 EVX evmwsmiaa
evmwsmian 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 1 1 0 0 1 EVX evmwsmian
evmwssf 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 1 0 0 1 1 EVX evmwssf
evmwssfa 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 1 0 0 1 1 EVX evmwssfa
evmwssfaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 1 0 0 1 1 EVX evmwssfaa
evmwssfan 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 1 0 0 1 1 EVX evmwssfan
evmwumi 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 0 1 1 0 0 0 EVX evmwumi
evmwumia 0 0 0 1 0 0 rD rA rB 1 0 0 0 1 1 1 1 0 0 0 EVX evmwumia
evmwumiaa 0 0 0 1 0 0 rD rA rB 1 0 1 0 1 0 1 1 0 0 0 EVX evmwumiaa
evmwumian 0 0 0 1 0 0 rD rA rB 1 0 1 1 1 0 1 1 0 0 0 EVX evmwumian
evnand 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 1 1 1 0 EVX evnand
evneg 0 0 0 1 0 0 rD rA /// 0 1 0 0 0 0 0 1 0 0 1 EVX evneg
evnor 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 1 0 0 0 EVX evnor
evor 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 0 1 1 1 EVX evor
evorc 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 1 0 1 1 EVX evorc
evrlw 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 1 0 0 0 EVX evrlw
evrlwi 0 0 0 1 0 0 rD rA UIMM 0 1 0 0 0 1 0 1 0 1 0 EVX evrlwi
evrndw 0 0 0 1 0 0 rD rA UIMM 0 1 0 0 0 0 0 1 1 0 0 EVX evrndw
evsel 0 0 0 1 0 0 rD rA rB 0 1 0 0 1 1 1 1 crfS EVX evsel
evslw 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 0 1 0 0 EVX evslw
evslwi 0 0 0 1 0 0 rD rA UIMM 0 1 0 0 0 1 0 0 1 1 0 EVX evslwi
evsplatfi 0 0 0 1 0 0 rD SIMM /// 0 1 0 0 0 1 0 1 0 1 1 EVX evsplatfi
evsplati 0 0 0 1 0 0 rD SIMM /// 0 1 0 0 0 1 0 1 0 0 1 EVX evsplati
evsrwis 0 0 0 1 0 0 rD rA UIMM 0 1 0 0 0 1 0 0 0 1 1 EVX evsrwis
evsrwiu 0 0 0 1 0 0 rD rA UIMM 0 1 0 0 0 1 0 0 0 1 0 EVX evsrwiu
evsrws 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 0 0 0 1 EVX evsrws
evsrwu 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 1 0 0 0 0 0 EVX evsrwu
evstdd 0 0 0 1 0 0 rD rA UIMM 1 0 1 1 0 0 1 0 0 0 0 1 EVX evstdd
evstddx 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 0 0 0 0 0 EVX evstddx
evstdh 0 0 0 1 0 0 rS rA UIMM 1 0 1 1 0 0 1 0 0 1 0 1 EVX evstdh
evstdhx 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 0 0 1 0 0 EVX evstdhx
evstdw 0 0 0 1 0 0 rS rA UIMM 1 0 1 1 0 0 1 0 0 0 1 1 EVX evstdw
evstdwx 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 0 0 0 1 0 EVX evstdwx
evstwhe 0 0 0 1 0 0 rS rA UIMM 3 0 1 1 0 0 1 1 0 0 0 1 EVX evstwhe
evstwhex 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 1 0 0 0 0 EVX evstwhex
evstwho 0 0 0 1 0 0 rS rA UIMM 3 0 1 1 0 0 1 1 0 1 0 1 EVX evstwho
evstwhox 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 1 0 1 0 0 EVX evstwhox
evstwwe 0 0 0 1 0 0 rS rA UIMM 3 0 1 1 0 0 1 1 1 0 0 1 EVX evstwwe
evstwwex 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 1 1 0 0 0 EVX evstwwex
evstwwo 0 0 0 1 0 0 rS rA UIMM 3 0 1 1 0 0 1 1 1 1 0 1 EVX evstwwo
evstwwox 0 0 0 1 0 0 rS rA rB 0 1 1 0 0 1 1 1 1 0 0 EVX evstwwox
evsubfsmiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 1 0 1 1 EVX evsubfsmiaaw
evsubfssiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 0 0 1 1 EVX evsubfssiaaw
evsubfumiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 1 0 1 0 EVX evsubfumiaaw
evsubfusiaaw 0 0 0 1 0 0 rD rA /// 1 0 0 1 1 0 0 0 0 1 0 EVX evsubfusiaaw
evsubfw 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 0 0 1 0 0 EVX evsubfw
evsubifw 0 0 0 1 0 0 rD UIMM rB 0 1 0 0 0 0 0 0 1 1 0 EVX evsubifw
evxor 0 0 0 1 0 0 rD rA rB 0 1 0 0 0 0 1 0 1 1 0 EVX evxor
EOF
}

#
# Extract the mnemonic and format of one or more instructions.
#
extract_mnemonic_format()
{
    cut -d' ' -f1,8,9,10
}

#
# Extract the format of one or more instructions.
#
extract_format()
{
    cut -d' ' -f8,9,10
}

#
# List all possible instruction formats.
#
list_formats()
{
    list_insn |extract_format |sort -u
}

#
# Match all instructions for a given format.
#
match_insn()
{
    local match=${1}

    list_insn |extract_mnemonic_format |grep "${match}$" |cut -d' ' -f1
}

#
# Generate the test macro invocation for instructions matching a given format.
#
generate_test_insn()
{
    local format=${1}
    local macro=${2}

    for insn in $(match_insn "${format}"); do
        echo "${macro}(${insn}, 0) \\"
    done
}

if [ "${1}" = "--list-formats" ]; then
    list_formats
else
    generate_test_insn "rD rA ///"   "TEST_INSN_RD_RA     "
    generate_test_insn "rD /// rB"   "TEST_INSN_RD_RA     "
    generate_test_insn "rD rA rB"    "TEST_INSN_RD_RA_RB  "
    generate_test_insn "rS rA rB"    "TEST_INSN_RD_RA_RB  "
    generate_test_insn "rD rA UIMM"  "TEST_INSN_RD_RA_UIMM"
    generate_test_insn "rD rA UIMM1" "TEST_INSN_RD_RA_UIMM"
    generate_test_insn "rD rA UIMM2" "TEST_INSN_RD_RA_UIMM"
    generate_test_insn "rD rA UIMM3" "TEST_INSN_RD_RA_UIMM"
    generate_test_insn "rS rA UIMM"  "TEST_INSN_RD_RA_UIMM"
    generate_test_insn "rD SIMM ///" "TEST_INSN_RD_SIMM   "
    generate_test_insn "rD UIMM rB"  "TEST_INSN_RD_UIMM_RB"
    generate_test_insn "crfD rA rB"  "TEST_INSN_CRD_RA_RB "
fi
